
#include <SPI.h>
#include <RH_RF95.h>

/* ESP32 feather w/wing */
#define RFM95_RST     27
#define RFM95_CS      15
#define RFM95_INT     14

// Change to 434.0 or other frequency, must match RX's freq!
#define RF95_FREQ 915.0

// Singleton instance of the radio driver
RH_RF95 rf95(RFM95_CS, RFM95_INT);

void setup() {

  pinMode(RFM95_RST, OUTPUT);
  digitalWrite(RFM95_RST, HIGH);

  Serial.begin(230400);
  while (!Serial) {
    delay(1);
  }
  delay(100);

  // manual reset
  digitalWrite(RFM95_RST, LOW);
  delay(10);
  digitalWrite(RFM95_RST, HIGH);
  delay(10);

  while (!rf95.init()) {
    Serial.println("LoRa radio init failed");
    Serial.println("Uncomment '#define SERIAL_DEBUG' in RH_RF95.cpp for detailed debug info");
  }
  //Serial.println("LoRa radio init OK!");

  // Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM
  if (!rf95.setFrequency(RF95_FREQ)) {
    Serial.println("setFrequency failed");
  }
  //Serial.print("Set Freq to: "); Serial.println(RF95_FREQ);

  // Defaults after init are 434.0MHz, 13dBm, Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on
  // The default transmitter power is 13dBm, using PA_BOOST.
  // If you are using RFM95/96/97/98 modules which uses the PA_BOOST transmitter pin, then
  // you can set transmitter powers from 5 to 23 dBm:
  rf95.setTxPower(23, false);
}

void loop()
{
  if (rf95.available())
  {
    // Should be a message for us now
    uint8_t buf[RH_RF95_MAX_MESSAGE_LEN];
    uint8_t len = sizeof(buf);

    if (rf95.recv(buf, &len)) {
      String recvd = (char*)buf;
      int excIndex = recvd.indexOf('!');
      int recvdLen = recvd.length();
      recvd.remove(excIndex, recvdLen - excIndex);
      Serial.print(recvd);
      Serial.print(", ");
      Serial.println(rf95.lastRssi(), DEC);
    }

    if (Serial.available() > 0) {
      String incomingString = Serial.readStringUntil('!');
      if (incomingString == "LAUNCH") {
        uint8_t data[] = "LAUNCH!";
        rf95.send(data, sizeof(data));
        rf95.waitPacketSent();
        //Serial.println("Launch Command Sent");
      }
      if (incomingString == "ABORT") {
        uint8_t data[] = "ABORT!";
        rf95.send(data, sizeof(data));
        rf95.waitPacketSent();
        //Serial.println("Abort Command Sent");
      }
      if (incomingString == "EOM") {
        uint8_t data[] = "EOM!";
        rf95.send(data, sizeof(data));
        rf95.waitPacketSent();
        //Serial.println("Abort Command Sent");
      }
    }
  }
}
